(function(){
	var ns = __namespace( "Astro.Blog.AstroEdit" );

	/** @type {System.Debug} */
	var debug                            = __import( "System.Debug" );
	/** @type {Components.MessageBox} */
	var MessageBox                       = __import( "Components.MessageBox" );
	/** @type {Dandelion} */
	var Dand                             = __import( "Dandelion" );
	/** @type {Dandelion.IDOMElement} */
	var IDOMElement                      = __import( "Dandelion.IDOMElement" );
	/** @type {Astro.Blog.Config} */
	var Config                           = __import( "Astro.Blog.Config" );

	var postData                         = __import( "System.Net.postData" );

	/** @param {Astro.Blog.AstroEdit.Article} */
	var Tag = function ( ae_article, tagCountUri )
	{
		var Article = ns[ NS_INVOKE ]( "Article" );

		if ( !( ae_article instanceof Article ) )
			return false;

		/** @type {_AstConf_.AstroEdit} */
		var a_conf = Config.get( "AstroEdit" );

		var ae_document = ae_article;

		var ae_addTag = Dand.id( "ae_addTags" );
		var ae_tags = Dand.id( "ae_tags" );

		// Store tags for future use
		var tags = {};

		////// Handlers
		ae_addTag.onclick = function ()
		{

			// Input fields
			var ae_tagInput;

			// Popup MessageBox
			new MessageBox(
				"Add new tag(s)"
				, Dand.wrape([
					Dand.wrapc(
						"v_instruction flsf"
						, "Tags are separated by \\n. (existing tags will be ignored.)"
					)
					, ae_tagInput = Dand.wrap( "textarea", null, "v_snippet_input" )
				])
				, "OK", "Cancel"
				, parseTags.bind( ae_tagInput )
			).show();
		};

		var	deleteTag = function (e)
		{
			var p = { tag: this.nodeValue, stage: this.parentNode };
			postData( tagCountUri, p, confirmDelete.bind( p ), serverFailed );
			e.stopPropagation();
		};

		var confirmDelete = function (obj)
		{
			new MessageBox(
				"Delete Tag"
				, [
					Dand.wrapc("ae_blockswitch",
						[
							Dand.textNode( "Are you sure you want to delete " )
							, Dand.wrap( "span", null, "tag_active", Dand.textNode( this.tag ) )
							, Dand.textNode( " ?." )
						]
					)
					, obj.count > 0
					? Dand.wrape([
						Dand.textNode( "This will affect " )
						, Dand.wrap(
							"span", null, "ae_affected_count", Dand.textNode( obj.count )
							)
						, Dand.textNode(" article(s).")
					])
					: null
				]
				, "Delete", "No"
				, doDelete.bind(this)
			).show();
		};

		var doDelete = function( confirmed )
		{
			if ( confirmed )
			{
				postData(
					a_conf.paths.set_tag
					, { tag: this.tag, del: 1 }
					, deleteSuccess.bind( this )
					, serverFailed
				);
			}
		};

		var deleteSuccess = function(obj)
		{
			// Remove element
			this.stage.parentNode.removeChild(this.stage);
			// Delete reference
			delete tags[this.tag];
		};

		var parseTags = function ()
		{
			// Filter invalid tags
			var ntags = this.value.trim().match(/[^\n]+/g), d, e;
			for (var i in ntags)
			{
				if(!tags[(i = ntags[i]).toLowerCase()])
				{
					tags[i] = Dand.wrap("span", null, "tag_active",
						[d = Dand.wrap("span", null, "ae_tag_delete", "\u00D7"), e = Dand.textNode(i)]
					);
					// Seperator
					ae_tags.appendChild(Dand.textNode("	"));
					ae_tags.appendChild(tags[i]);

					tags[i].onclick = function() { toggleTag(this) }.bind(tags[i]);
					d.onclick = deleteTag.bind(e);

				}
				else
				{
					debug.Info("Tag exist: " + i);
					// Hightlight these tags
					tags[i.toLowerCase()].className = "tag_active";
				}
			}

		};

		var toggleTag = function (tag)
		{
			var isEnabled = tag.className == "tag_active" ? true : false;
			tag.className = isEnabled ? "ae_disabled" : "tag_active";
		};

		var serverFailed = function (obj) { };



		var ctag, ae_children = ae_tags.childNodes;
		for (var i in ae_children)
		{
			if((ctag = ae_children[i]).nodeType == 1)
			{
				tags[ctag.lastChild.nodeValue.toLowerCase()] = ctag;
				IDOMElement( ctag ).addEventListener( "Click", function() { toggleTag( this ); }.bind( ctag ) );
				// Bind the last child (x button) to firstChild (textNode: tagname)
				IDOMElement( ctag.firstChild ).addEventListener( "Click", deleteTag.bind( ctag.lastChild ) );
			}
		}

		this.getTags = function ()
		{
			// Compile tag list
			var tlist = [];

			// Return names
			for (var i in tags)
			{
				if(tags[i].className == "tag_active")
					tlist[tlist.length] = tags[i].lastChild.nodeValue;
			}

			return tlist.join("\n");
		}

		this.setTags = function (tagList)
		{
			// Disable all tags first
			for (var i in tags) tags[i].className = "ae_disabled";

			// Set tags from tagList
			for (i in tagList)
				if (tagList[i]) tags[tagList[i].toLowerCase()].className = "tag_active";
		}

		ae_document.invoke(this);
	}

	Tag.prototype.getTags = function () {	};
	Tag.prototype.setTags = function (tagList) {	};

	ns[ NS_EXPORT ]( EX_CLASS, "Tag", Tag );

})();
